function [M, N1, F, x, K,D,B] = pillar(data, cutoff, int, blocks, alpha, bita, samplefrequency)

% [M, N, F, x, K] = pillar(data, int, blocks) automatically detects the time point of seizure
% onset in depth EEG signals. 'data' is the EEG signal. 'cutoff' denotes the cutoff 
% frequency for the lowpass Gaussian filter used to preprocess the raw data mostly to 
% remove muscle contraction artifacts. 'int' is the length of time 
% window. 'blocks' is the number of windows each with length int, to be tested for 
% the duration of the seizure after its onset. 'alpha' is a positive
% fraction. 'bita' is a positive proper fraction. 'samplefrequency' is the sampling
% frequency of the digitization of the EEG. If there is no seizure it will display
% 'No seizure is detected' and quit. Else it will return a plot with three windows.
% The top one is a variation diagram, beginning of which is the beginning of
% the seizure and the end is signified at the start of a low valued more or less 
% uniform platue. The middle window gives the exponential of a normalized absolute
% value of the double derivative of the original EEG signal. The bottom window 
% plots the EEG in time domain.
% Written by Kaushik Majumdar, September 2009.(2010)

% Output value initialization (partially).

M = 0;
N1 = 0;

% End of initialization (partially, more will follow further down).

% Preprocessing by lowpass Gaussian filter with the cutoff frequency specified.

Y = filter(gaussfiltcoef(samplefrequency, cutoff), 1, data);

% Suppression of isolated, possibly nonseizure spikes.

if bita < 0 || bita > 1
    disp('"bita" must be a positive proper fraction.');
    return;
end

Y1 = find(abs(Y) >= bita*max(abs(Y)));

for i = 1 : length(Y1)
    Y(Y1(i)) = 0;
end

X = Y;

% End of suppression.

% End of preprocessing.

% Normalized exponential of the absolute value of the derivative of the
% preprocessed data.

D = exp((abs(diff(diff(X))))/100000); % 1/100000 is the normalization constant.
% D = exp((abs(diff(X)))/100000);

k = length(D);
A = zeros(1,int);
B = zeros(k - int,1);
mm = zeros(blocks,1);

if floor(int) ~= int || int <= 100
    disp('"int" must be an integer greater than 100.');
    return;
    
else

% Windowed variance of D with window length equal to int.
    
for i = 1 : 100 : k - int
    for j = 1 : int
        A(1,j) = D(i + j - 1);
    end
%        B(i) = B(i) - mean(B(i)).*ones(length(B(i)),1);
        B(i) = var(A(1,:));
end

% End of variance calculation across all the windows.

% Selection of the window with variance less than that of the maximum
% variance by a fraction alpha.

if alpha <= 0 || alpha >= 1
    disp('"alpha" must be a positive proper fraction.');
    return;
else

M1 = find(B >= (1 - alpha)*max(B));

[C] = winvar1(abs(data),int);

M2 = find(C >= (1 - alpha)*max(C));

if abs(M1(1) - M2(1)) <= 15000
    if M1(1) <= M2(1)
        M = M1(1);
    else
        M = M2(1);
    end
else
    M = M1(1);
end

% End of window selection.

N = M + int;

end

disp(max(B));
disp(max(C));

% Start of calculation for the duration of the seizure.

if blocks*int > k - M
    
    disp('Requested number of points exceeds the available number of points.');
    return;
    
else
    
   for n = 1 : blocks
    mm(n) = var(D([N + (n - 1)*int : N + n*int]));
   end
    
end

n1 = find(mm == min(mm));
N1 = M + int*n1(1);

% End of duration calculation.

% Start of (patient specific) thresholding for seizure identification.

if max(B) < 1.32*10.^-6 || max(C) < 1.84*10.^5 || max(B) > 3.28*10.^-4 || max(C) > 8.45*10.^7
    disp('No seizure is detected.');
%    return;
end
    
% End of thresholding.

% First testing of the identified blocks for seizure.

disp(abs(max(B) - mm(1)));

if abs(max(B) - mm(1)) < 1.49*10.^-6 || abs(max(B) - mm(1)) > 3.14*10.^-4
    disp('No seizure is detected.');
%    return;
end

% End of first testing.

% Second testing of the iddentified blocks for seizure.

E = B([M - int*2 : N1]);
F = find(E >= (3/4)*max(E));
x = find(E == max(E));

if abs(floor(mean(F)) - x(1)) > 1950 || floor(std(F)) > 4138
    disp('No seizure is detected.');
%    return;
end

% End of second testing.

% Third testing of the identified blocks for seizure.

DE = exp(abs(diff(diff(E)))/0.001);
K = find(DE >= 0.999*max(DE));
disp('Display max(DE):');
disp(max(DE));
if length(K) > 40 || max(DE) < 1.002 || max(DE) > 10
    disp('No seizure is detected.');
%    return;
end

% End of third testing.

% Start of demarcation of seizure zone in the time plot of the EEG.
    
    disp('Seizure begins at (in seconds) after the recording started:');
    disp(floor(M/samplefrequency));
    disp('Seizure ends at (in seconds) after the recording started:');
    disp(ceil(N1/samplefrequency));

end

% End of demarcation.

% Plot of the EEG as well as the duration determination graph and seizure
% pillar visible in the plot of D.

subplot(4,1,1);
plot(mm);
title('A tentative measure of duration of the seizure.');
xlabel('Number of blocks each with the length of the specified window.');
subplot(4,1,2);
plot(DE);
title('Testing of the identified patch for seizure.');
subplot(4,1,3);
plot(D);
title('Normalized exponential of absolute value of double differentiation of the EEG.');
subplot(4,1,4);
plot(data);
line([M M], [-2*max(abs(data)) 2*max(abs(data))]);
line([N1 N1], [-2*max(abs(data)) 2*max(abs(data))]);
title('Plot of the raw EEG with the seizure part demarcated.');
xlabel('Time');

end